import typing
import pyhornedowl.pyhornedowl as pho
import os
import re

os.makedirs("pyhornedowl/model", exist_ok=True)

with open("pyhornedowl/__init__.py", "w") as f:
    pyhornedowl_members = [k for k, v in pho.__dict__.items() if isinstance(v, type) or callable(v)]

    f.write("from __future__ import annotations\n")
    f.write("from .pyhornedowl import ")
    f.write(", ".join(pyhornedowl_members))

    f.write("\n\n")
    f.write("__all__ = [")
    f.write(", ".join([f'"{n}"' for n in pyhornedowl_members]))
    f.write("]\n")

with open("pyhornedowl/__init__.pyi", "w") as f:
    f.write("import typing\nfrom typing import *\n\n")
    f.write("import model\n")
    f.write("\n")

    for name, entry in pho.__dict__.items():
        if isinstance(entry, type):
            f.write(f"class {name}:\n")

            for member_name, member in entry.__dict__.items():
                if member_name.startswith("_"):
                    continue

                if hasattr(member, "__doc__"):
                    doc = member.__doc__
                    if doc is not None:
                        lines = doc.splitlines()
                        if len(lines) > 2:
                            sign = lines[0]

                            f.write(f"    def {sign}:\n")
                            doc = "\n".join([f"        {l}" for l in lines[2:]])
                            f.write(f'        """\n{doc}\n        """\n        ...\n\n')

        if callable(entry):
            if hasattr(entry, "__doc__"):
                doc = entry.__doc__
                if doc is not None:
                    lines = doc.splitlines()
                    if len(lines) > 2:
                        sign = lines[0]

                        f.write(f"def {sign}:\n")
                        doc = "\n".join([f"    {l}" for l in lines[2:]])
                        f.write(f'    """\n{doc}\n    """\n     ..\n\n')

            f.write("\n")


with open("pyhornedowl/py.typed", "w"):
    pass


with open("pyhornedowl/model/__init__.py", "w") as f:
    f.write("from ..pyhornedowl import model\n")
    f.write("import typing\n\n")

    al = []

    for name, entry in pho.model.__dict__.items():
        if not isinstance(entry, type) and not type(entry) == typing._UnionGenericAlias:
            continue

        f.write(f"{name} = model.{name}\n")
        al.append(name)

    # module_pyi = pyhornedowl.model.__pyi__()

    # al += [l.split("=")[0].strip() for l in module_pyi.split("\n") if len(l.strip()) > 0]    

    # f.write(module_pyi)
    f.write("\n")

    f.write(f"__all__ = {al}")

with open("pyhornedowl/model/__init__.pyi", "w") as f:
    f.write("import typing\n")
    f.write("from typing import *\n\n")

    for name, entry in pho.model.__dict__.items():
        if isinstance(entry, type):
            if "__pyi__" in entry.__dict__:
                pyi: str = entry.__pyi__()
                pyi = re.sub(r"    from: .*$|from: .*?[,)]", "", pyi)
                f.write(pyi)
            else:
                f.write(f"class {name}:\n")

                for attr_name, attr in entry.__dict__.items():
                    if attr_name.startswith("_") or attr_name == "from":
                        continue

                    f.write(f"    {attr_name}: Any\n")

                f.write("    ...\n")
        elif type(entry) == typing._UnionGenericAlias:
            f.write(f"{name} = {str(entry).replace('pyhornedowl.model.', '')}")
        else:
            continue

        f.write("\n")
    
    # f.write(pyhornedowl.model.__pyi__())
    f.write("\n")



from __future__ import annotations
from .pyhornedowl import PyIndexedOntology, open_ontology, get_descendants, get_ancestors

__all__ = ["PyIndexedOntology", "open_ontology", "get_descendants", "get_ancestors"]



import json

with open("hornedowldoc/model.json") as f:
    data = json.load(f)

docs = dict([(x['name'], x['docs']) for x in data["index"].values() if 
             x['name'] is not None and 
             x['docs'] is not None and 
             ('struct' in x['inner'] or 'enum' in x['inner'] or 'variant' in x['inner'])])

lines = [ f'    ({n}) => {{ {json.dumps(d)} }};' for n, d in docs.items()]
lines.append("    ($t:ident) => {\"\"}")
         
with open("src/doc.rs", "w") as f:
    f.write('macro_rules! doc (\n'+ "\n".join(lines) +'\n);\n')


import typing
from typing import *

class Class:
    first: IRI
    def __init__(self, first: IRI):
        ...
    ...

class ObjectIntersectionOf:
    first: typing.List[ClassExpression]
    def __init__(self, first: typing.List[ClassExpression]):
        ...
    ...

class ObjectUnionOf:
    first: typing.List[ClassExpression]
    def __init__(self, first: typing.List[ClassExpression]):
        ...
    ...

class ObjectComplementOf:
    first: ClassExpression
    def __init__(self, first: ClassExpression):
        ...
    ...

class ObjectOneOf:
    first: typing.List[Individual]
    def __init__(self, first: typing.List[Individual]):
        ...
    ...

class ObjectSomeValuesFrom:
    ope: ObjectPropertyExpression
    bce: ClassExpression
    def __init__(self, ope: ObjectPropertyExpression, bce: ClassExpression):
        ...
    ...

class ObjectAllValuesFrom:
    ope: ObjectPropertyExpression
    bce: ClassExpression
    def __init__(self, ope: ObjectPropertyExpression, bce: ClassExpression):
        ...
    ...

class ObjectHasValue:
    ope: ObjectPropertyExpression
    i: Individual
    def __init__(self, ope: ObjectPropertyExpression, i: Individual):
        ...
    ...

class ObjectHasSelf:
    first: ObjectPropertyExpression
    def __init__(self, first: ObjectPropertyExpression):
        ...
    ...

class ObjectMinCardinality:
    n: int
    ope: ObjectPropertyExpression
    bce: ClassExpression
    def __init__(self, n: int, ope: ObjectPropertyExpression, bce: ClassExpression):
        ...
    ...

class ObjectMaxCardinality:
    n: int
    ope: ObjectPropertyExpression
    bce: ClassExpression
    def __init__(self, n: int, ope: ObjectPropertyExpression, bce: ClassExpression):
        ...
    ...

class ObjectExactCardinality:
    n: int
    ope: ObjectPropertyExpression
    bce: ClassExpression
    def __init__(self, n: int, ope: ObjectPropertyExpression, bce: ClassExpression):
        ...
    ...

class DataSomeValuesFrom:
    dp: DataProperty
    dr: DataRange
    def __init__(self, dp: DataProperty, dr: DataRange):
        ...
    ...

class DataAllValuesFrom:
    dp: DataProperty
    dr: DataRange
    def __init__(self, dp: DataProperty, dr: DataRange):
        ...
    ...

class DataHasValue:
    dp: DataProperty
    l: Literal
    def __init__(self, dp: DataProperty, l: Literal):
        ...
    ...

class DataMinCardinality:
    n: int
    dp: DataProperty
    dr: DataRange
    def __init__(self, n: int, dp: DataProperty, dr: DataRange):
        ...
    ...

class DataMaxCardinality:
    n: int
    dp: DataProperty
    dr: DataRange
    def __init__(self, n: int, dp: DataProperty, dr: DataRange):
        ...
    ...

class DataExactCardinality:
    n: int
    dp: DataProperty
    dr: DataRange
    def __init__(self, n: int, dp: DataProperty, dr: DataRange):
        ...
    ...

class Datatype:
    first: IRI
    def __init__(self, first: IRI):
        ...
    ...

class DataIntersectionOf:
    first: typing.List[DataRange]
    def __init__(self, first: typing.List[DataRange]):
        ...
    ...

class DataUnionOf:
    first: typing.List[DataRange]
    def __init__(self, first: typing.List[DataRange]):
        ...
    ...

class DataComplementOf:
    first: DataRange
    def __init__(self, first: DataRange):
        ...
    ...

class DataOneOf:
    first: typing.List[Literal]
    def __init__(self, first: typing.List[Literal]):
        ...
    ...

class DatatypeRestriction:
    first: Datatype
    second: typing.List[FacetRestriction]
    def __init__(self, first: Datatype, second: typing.List[FacetRestriction]):
        ...
    ...

class SimpleLiteral:
    literal: str
    def __init__(self, literal: str):
        ...
    ...

class LanguageLiteral:
    literal: str
    lang: str
    def __init__(self, literal: str, lang: str):
        ...
    ...

class DatatypeLiteral:
    literal: str
    datatype_iri: IRI
    def __init__(self, literal: str, datatype_iri: IRI):
        ...
    ...

class ObjectProperty:
    first: IRI
    def __init__(self, first: IRI):
        ...
    ...

class InverseObjectProperty:
    first: ObjectProperty
    def __init__(self, first: ObjectProperty):
        ...
    ...

class AnnotatedAxiom:
    axiom: Axiom
    ann: typing.Set[Annotation]
    def __init__(self, axiom: Axiom, ann: typing.Set[Annotation]):
        ...
    ...

class Annotation:
    ap: AnnotationProperty
    av: AnnotationValue
    def __init__(self, ap: AnnotationProperty, av: AnnotationValue):
        ...
    ...

class AnnotationAssertion:
    subject: AnnotationSubject
    ann: Annotation
    def __init__(self, subject: AnnotationSubject, ann: Annotation):
        ...
    ...

class AnnotationProperty:
    first: IRI
    def __init__(self, first: IRI):
        ...
    ...

class AnnotationPropertyDomain:
    ap: AnnotationProperty
    iri: IRI
    def __init__(self, ap: AnnotationProperty, iri: IRI):
        ...
    ...

class AnnotationPropertyRange:
    ap: AnnotationProperty
    iri: IRI
    def __init__(self, ap: AnnotationProperty, iri: IRI):
        ...
    ...

class AnonymousIndividual:
    first: str
    def __init__(self, first: str):
        ...
    ...

class AsymmetricObjectProperty:
    first: ObjectPropertyExpression
    def __init__(self, first: ObjectPropertyExpression):
        ...
    ...

class ClassAssertion:
    ce: ClassExpression
    i: Individual
    def __init__(self, ce: ClassExpression, i: Individual):
        ...
    ...

class DataProperty:
    first: IRI
    def __init__(self, first: IRI):
        ...
    ...

class DataPropertyAssertion:
    dp: DataProperty
    from: Individual
    to: Literal
    def __init__(self, dp: DataProperty,  to: Literal):
        ...
    ...

class DataPropertyDomain:
    dp: DataProperty
    ce: ClassExpression
    def __init__(self, dp: DataProperty, ce: ClassExpression):
        ...
    ...

class DataPropertyRange:
    dp: DataProperty
    dr: DataRange
    def __init__(self, dp: DataProperty, dr: DataRange):
        ...
    ...

class DatatypeDefinition:
    kind: Datatype
    range: DataRange
    def __init__(self, kind: Datatype, range: DataRange):
        ...
    ...

class DeclareAnnotationProperty:
    first: AnnotationProperty
    def __init__(self, first: AnnotationProperty):
        ...
    ...

class DeclareClass:
    first: Class
    def __init__(self, first: Class):
        ...
    ...

class DeclareDataProperty:
    first: DataProperty
    def __init__(self, first: DataProperty):
        ...
    ...

class DeclareDatatype:
    first: Datatype
    def __init__(self, first: Datatype):
        ...
    ...

class DeclareNamedIndividual:
    first: NamedIndividual
    def __init__(self, first: NamedIndividual):
        ...
    ...

class DeclareObjectProperty:
    first: ObjectProperty
    def __init__(self, first: ObjectProperty):
        ...
    ...

class DifferentIndividuals:
    first: typing.List[Individual]
    def __init__(self, first: typing.List[Individual]):
        ...
    ...

class DisjointClasses:
    first: typing.List[ClassExpression]
    def __init__(self, first: typing.List[ClassExpression]):
        ...
    ...

class DisjointDataProperties:
    first: typing.List[DataProperty]
    def __init__(self, first: typing.List[DataProperty]):
        ...
    ...

class DisjointObjectProperties:
    first: typing.List[ObjectPropertyExpression]
    def __init__(self, first: typing.List[ObjectPropertyExpression]):
        ...
    ...

class DisjointUnion:
    first: Class
    second: typing.List[ClassExpression]
    def __init__(self, first: Class, second: typing.List[ClassExpression]):
        ...
    ...

class EquivalentClasses:
    first: typing.List[ClassExpression]
    def __init__(self, first: typing.List[ClassExpression]):
        ...
    ...

class EquivalentDataProperties:
    first: typing.List[DataProperty]
    def __init__(self, first: typing.List[DataProperty]):
        ...
    ...

class EquivalentObjectProperties:
    first: typing.List[ObjectPropertyExpression]
    def __init__(self, first: typing.List[ObjectPropertyExpression]):
        ...
    ...

class FacetRestriction:
    f: Facet
    l: Literal
    def __init__(self, f: Facet, l: Literal):
        ...
    ...

class FunctionalDataProperty:
    first: DataProperty
    def __init__(self, first: DataProperty):
        ...
    ...

class FunctionalObjectProperty:
    first: ObjectPropertyExpression
    def __init__(self, first: ObjectPropertyExpression):
        ...
    ...

class HasKey:
    ce: ClassExpression
    vpe: typing.List[PropertyExpression]
    def __init__(self, ce: ClassExpression, vpe: typing.List[PropertyExpression]):
        ...
    ...

class IRI:
    parse: Any
    ...

class Import:
    first: IRI
    def __init__(self, first: IRI):
        ...
    ...

class InverseFunctionalObjectProperty:
    first: ObjectPropertyExpression
    def __init__(self, first: ObjectPropertyExpression):
        ...
    ...

class InverseObjectProperties:
    first: ObjectProperty
    second: ObjectProperty
    def __init__(self, first: ObjectProperty, second: ObjectProperty):
        ...
    ...

class IrreflexiveObjectProperty:
    first: ObjectPropertyExpression
    def __init__(self, first: ObjectPropertyExpression):
        ...
    ...

class NamedIndividual:
    first: IRI
    def __init__(self, first: IRI):
        ...
    ...

class NegativeDataPropertyAssertion:
    dp: DataProperty
    from: Individual
    to: Literal
    def __init__(self, dp: DataProperty,  to: Literal):
        ...
    ...

class NegativeObjectPropertyAssertion:
    ope: ObjectPropertyExpression
    from: Individual
    to: Individual
    def __init__(self, ope: ObjectPropertyExpression,  to: Individual):
        ...
    ...

class ObjectPropertyAssertion:
    ope: ObjectPropertyExpression
    from: Individual
    to: Individual
    def __init__(self, ope: ObjectPropertyExpression,  to: Individual):
        ...
    ...

class ObjectPropertyDomain:
    ope: ObjectPropertyExpression
    ce: ClassExpression
    def __init__(self, ope: ObjectPropertyExpression, ce: ClassExpression):
        ...
    ...

class ObjectPropertyRange:
    ope: ObjectPropertyExpression
    ce: ClassExpression
    def __init__(self, ope: ObjectPropertyExpression, ce: ClassExpression):
        ...
    ...

class OntologyAnnotation:
    first: Annotation
    def __init__(self, first: Annotation):
        ...
    ...

class ReflexiveObjectProperty:
    first: ObjectPropertyExpression
    def __init__(self, first: ObjectPropertyExpression):
        ...
    ...

class SameIndividual:
    first: typing.List[Individual]
    def __init__(self, first: typing.List[Individual]):
        ...
    ...

class SubAnnotationPropertyOf:
    sup: AnnotationProperty
    sub: AnnotationProperty
    def __init__(self, sup: AnnotationProperty, sub: AnnotationProperty):
        ...
    ...

class SubClassOf:
    sup: ClassExpression
    sub: ClassExpression
    def __init__(self, sup: ClassExpression, sub: ClassExpression):
        ...
    ...

class SubDataPropertyOf:
    sup: DataProperty
    sub: DataProperty
    def __init__(self, sup: DataProperty, sub: DataProperty):
        ...
    ...

class SubObjectPropertyOf:
    sup: ObjectPropertyExpression
    sub: SubObjectPropertyExpression
    def __init__(self, sup: ObjectPropertyExpression, sub: SubObjectPropertyExpression):
        ...
    ...

class SymmetricObjectProperty:
    first: ObjectPropertyExpression
    def __init__(self, first: ObjectPropertyExpression):
        ...
    ...

class TransitiveObjectProperty:
    first: ObjectPropertyExpression
    def __init__(self, first: ObjectPropertyExpression):
        ...
    ...

class Facet:
    Length: Facet
    MinLength: Facet
    MaxLength: Facet
    Pattern: Facet
    MinInclusive: Facet
    MinExclusive: Facet
    MaxInclusive: Facet
    MaxExclusive: Facet
    TotalDigits: Facet
    FractionDigits: Facet
    LangRange: Facet

ClassExpression = typing.Union[ObjectIntersectionOf, ObjectUnionOf, ObjectComplementOf, ObjectOneOf, ObjectSomeValuesFrom, ObjectAllValuesFrom, ObjectHasValue, ObjectHasSelf, ObjectMinCardinality, ObjectMaxCardinality, ObjectExactCardinality, DataSomeValuesFrom, DataAllValuesFrom, DataHasValue, DataMinCardinality, DataMaxCardinality, DataExactCardinality, Class]
ObjectPropertyExpression = typing.Union[InverseObjectProperty, ObjectProperty]
SubObjectPropertyExpression = typing.Union[typing.List[typing.Union[InverseObjectProperty, ObjectProperty]], InverseObjectProperty, ObjectProperty]
Literal = typing.Union[SimpleLiteral, LanguageLiteral, DatatypeLiteral]
DataRange = typing.Union[DataIntersectionOf, DataUnionOf, DataComplementOf, DataOneOf, DatatypeRestriction, Datatype]
Individual = typing.Union[AnonymousIndividual, NamedIndividual]
PropertyExpression = typing.Union[InverseObjectProperty, ObjectProperty, DataProperty, AnnotationProperty]
AnnotationSubject = typing.Union[IRI, AnonymousIndividual]
AnnotationValue = typing.Union[SimpleLiteral, LanguageLiteral, DatatypeLiteral, IRI]
Axiom = typing.Union[OntologyAnnotation, Import, DeclareClass, DeclareObjectProperty, DeclareAnnotationProperty, DeclareDataProperty, DeclareNamedIndividual, DeclareDatatype, SubClassOf, EquivalentClasses, DisjointClasses, DisjointUnion, SubObjectPropertyOf, EquivalentObjectProperties, DisjointObjectProperties, InverseObjectProperties, ObjectPropertyDomain, ObjectPropertyRange, FunctionalObjectProperty, InverseFunctionalObjectProperty, ReflexiveObjectProperty, IrreflexiveObjectProperty, SymmetricObjectProperty, AsymmetricObjectProperty, TransitiveObjectProperty, SubDataPropertyOf, EquivalentDataProperties, DisjointDataProperties, DataPropertyDomain, DataPropertyRange, FunctionalDataProperty, DatatypeDefinition, HasKey, SameIndividual, DifferentIndividuals, ClassAssertion, ObjectPropertyAssertion, NegativeObjectPropertyAssertion, DataPropertyAssertion, NegativeDataPropertyAssertion, AnnotationAssertion, SubAnnotationPropertyOf, AnnotationPropertyDomain, AnnotationPropertyRange]




import typing
from typing import *

import model

class PyIndexedOntology:
    def get_id_for_iri(self, iri: str) -> Optional[str]:
        """
        Gets the ID of term by it IRI.
        
        If the term does not have an ID, `None` is returned.
        """
        ...

    def get_iri_for_id(self, id: str) -> Optional[str]:
        """
        Gets the IRI of a term by its ID.
        
        If the term does not have an IRI, `None` is returned.
        """
        ...

    def add_prefix_mapping(self, iriprefix: str, mappedid: str) -> None:
        """
        Adds the prefix `iriprefix`.
        """
        ...

    def set_label(self, iri: str, label: str) -> None:
        """
        Sets the label of a term by iri.
        
        Adds an or updates the `AnnotationAssertion` axiom for `rdfs:label`.
        """
        ...

    def get_iri_for_label(self, label: str) -> Optional[str]:
        """
        Returns the IRI of a term by its label if it exists.
        
        If the term does not have a label, `None` is returned.
        """
        ...

    def get_iri(self) -> Optional[str]:
        """
        Returns the ontology iri, if it exists.
        """
        ...

    def get_version_iri(self) -> Optional[str]:
        """
        Returns the ontologys version iri, if it exists.
        """
        ...

    def get_subclasses(self, iri: str) -> Set[str]:
        """
        Gets all subclasses of an entity.
        """
        ...

    def get_superclasses(self, iri: str) -> Set[str]:
        """
        Gets all superclasses of an entity.
        """
        ...

    def get_classes(self) -> Set[str]:
        """
        Returns the IRIs of all declared classes in the ontology.
        """
        ...

    def get_object_properties(self) -> Set[str]:
        """
        Returns the IRIs of all declared object properties in the ontology.
        """
        ...

    def get_annotation(self, class_iri: str, ann_iri: str) -> Optional[str]:
        """
        Gets the first annotated value for an entity and annotation property.
        
        Note: If there are multiple annotation axioms for the queried entity and annotation property,
        the order is neither necessarily the same as in the ontology neither is it stable.
        Get all annotation values with `PyIndexedOntology.get_annotations`.
        """
        ...

    def get_annotations(self, class_iri: str, ann_iri: str) -> List[str]:
        """
        Gets all annotated value for an entity and annotation property.
        
        Note: The order is neither necessarily the same as in the ontology neither is it stable.
        Get all annotation values with `PyIndexedOntology.get_annotations`.
        """
        ...

    def save_to_file(self, file_name: str) -> None:
        """
        Saves the ontology to disk in owx format.
        """
        ...

    def get_axioms_for_iri(self, iri: str) -> List[model.AnnotatedAxiom]:
        """
        Gets all axioms for an entity.
        """
        ...

    def get_axioms(self) -> List[model.AnnotatedAxioms]:
        """
        Returns all axioms of the ontology.
        """
        ...

    def add_axiom(self, ax: model.Axiom, annotations: Optional[List[model.Annotation]]) -> None:
        """
        Adds an axiom to the ontology with optional annotations.
        """
        ...

    def remove_axiom(self, ax: model.Axiom) -> None:
        """
        Removes an axiom from the ontology.
        """
        ...

    def iri(self, iri: str) -> model.IRI:
        """
        Creates an new IRI from string.
        
        Use this method instead of  `model.IRI.parse` if possible as it is more optimized using caches.
        """
        ...


def open_ontology(ontology: str) -> PyIndexedOntology:
    """
    Opens an ontology from a path or plain text.
    
    If `ontology` is a path, the file is loaded. Otherwise, `ontology` is interepreted as an ontology in either owx or owl format.
    Note: Only .owl and .owx files are currently supported.
    """
     ..


def get_descendants(onto: PyIndexedOntology, parent: str) -> Set[str]:
    """
    Gets all direct and indirect subclasses of an class.
    """
     ..


def get_ancestors(onto: PyIndexedOntology, child: str) -> Set[str]:
    """
    Gets all direct and indirect super classes of a class.
    """
     ..


